// Fichier Propulsion.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/




#include <p18f452.h>
#include "serial.h"
#include "Propulsion.h"
#include "Constante.h"
//#include "strategie_bleu.h"
#include "IO.h"


// --------------------------------------------------------------------------
/* Retourne vrai si un mouvement est en cours (la cible n'est pas atteinte et
 * le robot n'est pas bloqu)
*/
char IsMove_Propulsion(void)
{
	char State=Etat_Propulsion();
	return !(State=='a' || State=='g' || State=='o');
}	


// --------------------------------------------------------------------------
/* Avance (ou recule) de distance en mm
 * Arguments : 
 *  Distance  parcourir en ligne droite
 *  Vitesse de dplacement (si c'est positif on avance, sinon on recule)
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int ToutDroit_Propulsion (int Distance, int Vitesse)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='T';						// 'T' pour informer qu'il s'agit d'une commande ToutDroit
	Trame[1]=(Distance&0xFF00)>>8;		// Octet de poids fort de la distance en X
	Trame[2]=Distance&0x00FF;			// Octet de poids faible 
	Trame[3]=(Vitesse&0xFF00)>>8;	 	// Octet de poids fort de la vitesse de dplacement
	Trame[4]=Vitesse&0x00FF;			// Octet de poids faible 
	Trame[5]=0xFF;			 			// Octet de poids fort de la vitesse
	Trame[6]=0x00;						// Octet de poids faible 
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,30)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='T') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	
	return (0);	
}	







// --------------------------------------------------------------------------
/* Oriente le robot vers l'angle pass en paramtre
 * Arguments : 
 *  Orientation dsire du robot en radians
 *  Vitesse angulaire maximum de rotation
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Angle_Propulsion (float Angle, int VitesseMax)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	int Angle_Entier;					// Copie de l'angle en entier *1000
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	Angle_Entier=Angle*1000;			// Multiplie par 1000 avant l'envoi
	
	if (VitesseMax<0) 					// Vrifie que la vitesse est bien positive
		VitesseMax=-VitesseMax;
	
	Trame[0]='O';						// 'G' pour informer qu'il s'agit d'une commande Goto
	Trame[1]=(Angle_Entier&0xFF00)>>8;	// Octet de poids fort de l'angle
	Trame[2]=Angle_Entier&0x00FF;		// Octet de poids faible 
	Trame[3]=(VitesseMax&0xFF00)>>8;	// Octet de poids fort de la vitesse angulaire maximum
	Trame[4]=VitesseMax&0x00FF;;		// Octet de poids faible 
	Trame[5]=0;	 						// Octet de poids fort
	Trame[6]=0xFF;						// Octet de poids faible 
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}




// --------------------------------------------------------------------------
/* Envoie le robot vers le point X,Y
 * Arguments : 
 *  Position en X
 *  Position en Y 
 *  Vitesse de dplacement
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int GotoXY_Propulsion (int X, int Y, int V)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='G';						// 'G' pour informer qu'il s'agit d'une commande Goto
	Trame[1]=(X&0xFF00)>>8;				// Octet de poids fort de la position en X
	Trame[2]=X&0x00FF;					// Octet de poids faible 
	Trame[3]=(Y&0xFF00)>>8;	 			// Octet de poids fort de la position en Y
	Trame[4]=Y&0x00FF;					// Octet de poids faible 
	Trame[5]=(V&0xFF00)>>8;	 			// Octet de poids fort de la vitesse
	Trame[6]=V&0x00FF;					// Octet de poids faible 
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='G') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	
	return (0);
}






// --------------------------------------------------------------------------
/* Force la position courante du robot
 * Arguments : 
 *  Position en X
 *  Position en Y
 *  Orientation du robot (en radians * 1000)
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Force_Propulsion (int X, int Y, int A)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='F';						// 'D' pour informer qu'il s'agit d'une commande directe
	Trame[1]=(X&0xFF00)>>8;				// Octet de poids fort de la position en X
	Trame[2]=X&0x00FF;					// Octet de poids faible 
	Trame[3]=(Y&0xFF00)>>8;	 			// Octet de poids fort de la position en Y
	Trame[4]=Y&0x00FF;					// Octet de poids faible 
	Trame[5]=(A&0xFF00)>>8;		 		// Octet de poids fort de l'orientation
	Trame[6]=A&0x00FF;					// Octet de poids faible 
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='F') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}







// --------------------------------------------------------------------------
/* Stop la commande en cours et arrte les moteurs
 * Arguments : aucun 
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Direct_Propulsion (int VitG,int VitD)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='D';						// 'D' pour informer qu'il s'agit d'une commande directe
	Trame[1]=(VitG&0xFF00)>>8;			// Octet de poids fort de la commande du moteur gauche	
	Trame[2]=VitG&0x00FF;				// Octet de poids faible de la commande du moteur gauche
	Trame[3]=(VitD&0xFF00)>>8;	 		// Octet de poids fort de la commande du moteur droit
	Trame[4]=VitD&0x00FF;				// Octet de poids faible de la commande du moteur droit
	Trame[5]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)
	Trame[6]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)		
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='D') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}


// --------------------------------------------------------------------------
/* Commande les moteurs en mode direct asservi
 * Arguments : aucun 
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Direct_Asservi_Propulsion (int VitG,int VitD)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='B';						// 'D' pour informer qu'il s'agit d'une commande directe
	Trame[1]=(VitG&0xFF00)>>8;			// Octet de poids fort de la commande du moteur gauche	
	Trame[2]=VitG&0x00FF;				// Octet de poids faible de la commande du moteur gauche
	Trame[3]=(VitD&0xFF00)>>8;	 		// Octet de poids fort de la commande du moteur droit
	Trame[4]=VitD&0x00FF;				// Octet de poids faible de la commande du moteur droit
	Trame[5]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)
	Trame[6]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)		
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='B') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}

// --------------------------------------------------------------------------
/* Commande les moteurs en polaire : vitesse linaire et vitesse angulaire
 * Arguments :
 *  - Le premier paramtre est la vitesse linaire [-100,+100]
 *  - Le second paramtre est la vitesse angulaire [-50,+50]
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Polaire_Propulsion (int VitL,int VitA)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='P';						// 'P' pour informer qu'il s'agit d'une commande polaire
	Trame[1]=(VitL&0xFF00)>>8;			// Octet de poids fort de la vitesse linaire
	Trame[2]=VitL&0x00FF;				// Octet de poids faible de la vitesse linaire
	Trame[3]=(VitA&0xFF00)>>8;	 		// Octet de poids fort de la vitesse angulaire
	Trame[4]=VitA&0x00FF;				// Octet de poids faible de la vitesse angulaire
	Trame[5]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)
	Trame[6]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)		
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='P') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}






// --------------------------------------------------------------------------
/* Envoie une commande directe sur les moteurs
 * Arguments :
 *  - Le premier paramtre est la vitesse du moteur gauche [-1023,+1023]
 *  - Le second paramtre est la vitesse du moteur droit [-1023,+1023]
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractre reus ne corresponde pas aux caractre attendus
*/
int Stop_Propulsion (void)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='A';						// Envoi A_/|\_0	Drole de squence pour dire on arrte !
	Trame[1]='_';					
	Trame[2]='/';					
	Trame[3]='|';				 	
	Trame[4]='\\';					
	Trame[5]='_';				
	Trame[6]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)	
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,30)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='A') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}	






// --------------------------------------------------------------------------
/* Lit l'tat (tat + position + orientation) de la carte propulsion
 * Arguments :
 *  - Le premier paramtre est un pointeur vers l'tat de la carte
 *  - Le second paramtre est un pointeur vers la coordonne en X
 *  - Le troisime paramtre est un pointeur vers la coordonne en Y
 *  - Le dernier paramtre est un pointeur vers l'orientation
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Erreur de CRC
*/
int Info_Propulsion (int *S,int *X,int *Y,int *A)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='S';						// Envoi STETE?	(le A est remplacer pour limiter les risques d'erreurs)
	Trame[1]='T';					
	Trame[2]='E';					
	Trame[3]='T';				 	
	Trame[4]='E';					
	Trame[5]='?';				
	Trame[6]=0;							// 0 pour complter la trame (7 caractre par trame + le CRC)		
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,8,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[7]==CRC(Trame,7))			// Vrifie que le CRC de la trame reue est correct
	{
		*S=Trame[0];					// Isole l'tat courant de la carte
		*X=Trame[1]*256 + Trame[2];		// Reconstitue le X
		*Y=Trame[3]*256 + Trame[4];		// Reconstitue le Y
		*A=Trame[5]*256 + Trame[6];		// Reconstitue l'Angle
		return (0);						// Tout s'est merveilleusement bien pass !
	}
	return (-2);						// Erreur de CRC	
}	


// --------------------------------------------------------------------------
/* Lit l'tat  de la carte propulsion
 * Arguments : aucun
 * 
 * Paramtre renvoy : 
 *   Etat courant de la carte
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Erreur de CRC
*/
char Etat_Propulsion (void)
{
	int State,Tmp,Ret;
	Ret=Info_Propulsion (&State,&Tmp,&Tmp,&Tmp);		// Lit l'tat de la carte	
	if (Ret) return (Ret);								// Il y a une erreur ?
	return (State);	
}


// --------------------------------------------------------------------------
/* Retourne la position en X du robot
 * Arguments : aucun
 * 
 * Paramtre renvoy : 
 *   Position du robot en X
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Erreur de CRC
*/
int X_Propulsion (void)
{
	int X,Tmp,Ret;
	Ret=Info_Propulsion (&Tmp,&X,&Tmp,&Tmp);			// Lit l'tat de la carte	
	if (Ret) return (Ret);								// Il y a une erreur ?
	return (X);											// Retourne la coordonne en X
}



// --------------------------------------------------------------------------
/* Retourne la position en Y du robot
 * Arguments : aucun
 * 
 * Paramtre renvoy : 
 *   Position du robot en Y
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Erreur de CRC
*/
int Y_Propulsion (void)
{
	int Y,Tmp,Ret;
	Ret=Info_Propulsion (&Tmp,&Tmp,&Y,&Tmp);			// Lit l'tat de la carte	
	if (Ret) return (Ret);								// Il y a une erreur ?
	return (Y);											// Retourne la coordonne en Y
}


// --------------------------------------------------------------------------
/* Retourne l'orientation du robot
 * Arguments : aucun
 * 
 * Paramtre renvoy : 
 *   Orientation du robot (x1000)
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Erreur de CRC
*/
int A_Propulsion (void)
{
	int Angle,Tmp,Ret;
	Ret=Info_Propulsion (&Tmp,&Tmp,&Tmp,&Angle);			// Lit l'tat de la carte	
	if (Ret) return (Ret);									// Il y a une erreur ?
	return (Angle);											// Retourne la coordonne en Y
}

// --------------------------------------------------------------------------
/* Effectue un chek-up de la propulsion
 * Arrte les moteurs   								(erreur 1)
 * Force la position du robot  						 	(erreur 2)
 * Lit la nouvelle position 							(erreur 3)
 * Compare la nouvelle position  la position force	(erreur 4)
 * Force en position de dpart							(erreur 5)
 * Avance de 3,4 cm										(erreur 6)
 * S'oriente 				  						 	(erreur 7)
 * Arrte les moteur									(erreur 8)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Propulsion (unsigned char couleur)
{
	int S,X,Y, A;
	if (Stop_Propulsion())  					return 1;		// Arrte les moteurs					
	if (Force_Propulsion (-1,23,-456)) 			return 2;		// Force la position 
	if (Info_Propulsion (&S,&X,&Y,&A)) 			return 3;		// Lit la position du robot
	if (S!='a' || X!=-1 || Y!=23 | A!=-456)		return 4;		// Vrifie que la position force est la bonne			
	if (couleur==ROUGE)											// Si on joue rouge
	{
		if (Force_Propulsion(66,200,0))			return 5;		// Force la position de dpart
	}
	else														// si on joue bleu
	{
		if (Force_Propulsion(2934,200,3141))	return 5;		// Force la position de dpart
	}
	
	if (ToutDroit_Propulsion(34,5))				return 6;		// Avance de 3,4 cm
	while (IsMove_Propulsion());							    // Attend la fin du mouvement
	
	if (couleur==ROUGE){										// Si on joue rouge
		if (Angle_Propulsion (-0.320,5))		return 7;		// S'oriente en position de dpart
	}else{
		if (Angle_Propulsion (PI+0.315,5))		return 7;		// S'oriente en position de dpart
	}
	
	while (Etat_Propulsion()!='a' && Etat_Propulsion()!='o');	// Attend la fin du mouvement
	if (Stop_Propulsion())  					return 8;		// Arrte les moteurs	
	
	return 0;													// Tout s'est bien pass
}	



// --------------------------------------------------------------------------
/* Recalibrage du robot en X,Y et en A par rapport  la postion rel
 * Arguments : 
 *  DeltaX
 *  DeltaY
 *  DeltaA (en radians * 1000)
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
 *  -2 : Les caractres reus ne correspondent pas aux caractres attendus
*/
int Recalibrage_Robot (int dX, int dY, int dA)
{
	unsigned char Trame[8];				// Tableau qui va contenir la trame  envoyer
	
	Vide_Buffer_USART();				// Prpare la ligne srie pour la rception des caractres
	
	Trame[0]='R';						// 'R' pour informer qu'il s'agit d'un recalibrage du robot
	Trame[1]=(dX&0xFF00)>>8;				// Octet de poids fort de dX
	Trame[2]=dX&0x00FF;					// Octet de poids faible 
	Trame[3]=(dY&0xFF00)>>8;	 			// Octet de poids fort de dY
	Trame[4]=dY&0x00FF;					// Octet de poids faible 
	Trame[5]=(dA&0xFF00)>>8;		 		// Octet de poids fort de l'orientation
	Trame[6]=dA&0x00FF;					// Octet de poids faible 
	Trame[7]=CRC(Trame,7);				// Ajout du CRC	
	Envoyer_Trame (Trame,8);			// Envoie de la trame
	
	if (Recevoir_Trame(Trame,3,20)) 	// Reception de la trame avant le TimeOut (ici 20ms)
		return (-1); 					// TimeOut, on retourne -1 pour signifier l'erreur
			
	if (Trame[0]!='R') return -2;		// Ce n'est pas le message attendu
	if (Trame[1]!='O') return -2;		// Ce n'est pas le message attendu
	if (Trame[2]!='K') return -2;		// Ce n'est pas le message attendu
	
	return (0);
}