// Fichier Rouleaux.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <delays.h>			// Delais pour le check-up


#include "I2C_Master.h"		// Librairie pour la liaison srie
#include "Rouleaux.h"		// Entte de la librairie de gestion du robot



// --------------------------------------------------------------------------
/* Commande les rouleaux
 * 	SpeedL : moteur gauche (vitesse entre -128 et +127)
 * 	SpeedR : moteur droit (vitesse entre -128 et +127)
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleaux(char SpeedL, char SpeedR){
	int ret;
	ret = (I2C_Send(ADR_I2C_ROULEAU_GAUCHE,SpeedL));		// Retourne le rsultat de l'envoie sur l'I2C
	if (ret<0)
	return ret;
	
	return (I2C_Send(ADR_I2C_ROULEAU_DROIT,-SpeedR));		// Retourne le rsultat de l'envoie sur l'I2C
}	


// --------------------------------------------------------------------------
/* Commande le rouleau droit
 * le paramtre correspond  la vitesse  envoyer entre -128 et +127
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleau_Droit(char Speed) 
{
	return (I2C_Send(ADR_I2C_ROULEAU_DROIT,-Speed));		// Retourne le rsultat de l'envoie sur l'I2C
}	

// --------------------------------------------------------------------------
/* Mesure la vitesse du rouleau droit
 * La fonction mesure la vitesse du rouleau droit et retourne sa valeur
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Rouleau_Droit(void) 
{
	int Speed=I2C_Receive(ADR_I2C_ROULEAU_DROIT);		// Lit la vitesse via l'IC
	if (Speed==-1) 										// En cas de problme de transmission ..
		return (0x0FFF);								// .. on retourne 0x0FFF
	else												// si tout va bien
		return (-(char)Speed);							// on retourne la vitesse caste en char
}

// --------------------------------------------------------------------------
/* Effectue un chek-up du rouleau droit
 * Lance le rouleau en avant et vrifie la vitesse   (erreurs 1 et 2)
 * Arrte le moteur et vrifie qu'il est  l'arrt   (erreurs 3 et 4)
 * Lance le rouleau en arrire et vrifie la vitesse (erreurs 5 et 6)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Rouleau_Droit (void)
{
	if (Commande_Rouleau_Droit(CHECK_UP_VITESSE_ROULEAUX))  return 1; 		// Envoie +70 sur le rouleau
	Delay10KTCYx(CHECK_UP_DELAY);							
	if (Mesure_Rouleau_Droit()<CHECK_UP_MESURE_ROULEAUX) 	return 2;		// Mesure la vitesse et vrifie la cohrence
	if (Commande_Rouleau_Droit(0)) 							return 3;		// Arrte le moteur
	Delay10KTCYx(CHECK_UP_DELAY);
	if (Mesure_Rouleau_Droit()!=0)							return 4;		// Vrifie que le moteur est  l'arrt
	if (Commande_Rouleau_Droit(-CHECK_UP_VITESSE_ROULEAUX)) return 5;		// Envoie -70 sur le rouleau
	Delay10KTCYx(CHECK_UP_DELAY);
	if (Mesure_Rouleau_Droit()>-CHECK_UP_MESURE_ROULEAUX)  	return 6;		// Vrifie que le moteur tourne dans l'autre sens
	return 0;																// Tout s'est bien pass
}	


// --------------------------------------------------------------------------
/* Commande le rouleau gauche
 * le paramtre correspond  la vitesse  envoyer entre -128 et +127
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleau_Gauche(char Speed) 
{
	return (I2C_Send(ADR_I2C_ROULEAU_GAUCHE,Speed));		// Retourne le rsultat de l'envoie sur l'I2C
}	

// --------------------------------------------------------------------------
/* Mesure la vitesse du rouleau gauche
 * La fonction mesure la vitesse du rouleau droit et retourne sa valeur
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Rouleau_Gauche(void) 
{
	int Speed=I2C_Receive(ADR_I2C_ROULEAU_GAUCHE);		// Lit la vitesse via l'IC
	if (Speed==-1) 										// En cas de problme de transmission ..
		return (0x0FFF);								// .. on retourne 0x0FFF
	else												// si tout va bien
		return ((char)Speed);							// on retourne la vitesse caste en char
}

// --------------------------------------------------------------------------
/* Effectue un chek-up du rouleau Gauche
 * Lance le rouleau en avant et vrifie la vitesse   (erreurs 1 et 2)
 * Arrte le moteur et vrifie qu'il est  l'arrt   (erreurs 3 et 4)
 * Lance le rouleau en arrire et vrifie la vitesse (erreurs 5 et 6)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Rouleau_Gauche (void)
{
	if (Commande_Rouleau_Gauche(CHECK_UP_VITESSE_ROULEAUX))  return 1; 		// Envoie +70 sur le rouleau
	Delay10KTCYx(CHECK_UP_DELAY);							
	if (Mesure_Rouleau_Gauche()<CHECK_UP_MESURE_ROULEAUX) 	 return 2;		// Mesure la vitesse et vrifie la cohrence
	if (Commande_Rouleau_Gauche(0)) 						 return 3;		// Arrte le moteur
	Delay10KTCYx(CHECK_UP_DELAY);
	if (Mesure_Rouleau_Gauche()!=0)							 return 4;		// Vrifie que le moteur est  l'arrt
	if (Commande_Rouleau_Gauche(-CHECK_UP_VITESSE_ROULEAUX)) return 5;		// Envoie -70 sur le rouleau
	Delay10KTCYx(CHECK_UP_DELAY);
	if (Mesure_Rouleau_Gauche()>-CHECK_UP_MESURE_ROULEAUX)   return 6;		// Vrifie que le moteur tourne dans l'autre sens
	return 0;																// Tout s'est bien pass
}	